@@ -13,18 +13,21 @@ expected="<?xml version=\"1.0\"?>
     <test result=\"success\">
       <fn>ex_xml_output.c:8</fn>
       <id>test_pass</id>
+      <iteration>0</iteration>
       <description>Core</description>
       <message>Passed</message>
     </test>
     <test result=\"failure\">
       <fn>ex_xml_output.c:14</fn>
       <id>test_fail</id>
+      <iteration>0</iteration>
       <description>Core</description>
       <message>Failure</message>
     </test>
     <test result=\"error\">
       <fn>ex_xml_output.c:18</fn>
       <id>test_exit</id>
+      <iteration>0</iteration>
       <description>Core</description>
       <message>Early exit with return value 1</message>
     </test>
@@ -34,9 +37,31 @@ expected="<?xml version=\"1.0\"?>
     <test result=\"success\">
       <fn>ex_xml_output.c:26</fn>
       <id>test_pass2</id>
+      <iteration>0</iteration>
       <description>Core</description>
       <message>Passed</message>
     </test>
+    <test result=\"failure\">
+      <fn>ex_xml_output.c:32</fn>
+      <id>test_loop</id>
+      <iteration>0</iteration>
+      <description>Core</description>
+      <message>Iteration 0 failed</message>
+    </test>
+    <test result=\"success\">
+      <fn>ex_xml_output.c:32</fn>
+      <id>test_loop</id>
+      <iteration>1</iteration>
+      <description>Core</description>
+      <message>Passed</message>
+    </test>
+    <test result=\"failure\">
+      <fn>ex_xml_output.c:32</fn>
+      <id>test_loop</id>
+      <iteration>2</iteration>
+      <description>Core</description>
+      <message>Iteration 2 failed</message>
+    </test>
   </suite>
 </testsuites>"
 
